package com.galaxis.manatee.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.constant.DingTalkConstant;
import com.galaxis.manatee.constant.StringConstant;
import com.galaxis.manatee.dao.ChuanyunUserDao;
import com.galaxis.manatee.dao.DingTalkProcessInstanceDao;
import com.galaxis.manatee.dao.ChuanyunProjectDao;
import com.galaxis.manatee.entity.chuanyun.dto.*;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunProjectDO;
import com.galaxis.manatee.entity.chuanyun.bo.MaterialResendBO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunUserDO;
import com.galaxis.manatee.entity.ding.DingTalkProcessInstance;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.manager.ClawFeign;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/2/13 11:53 上午
 */
@Service
@Slf4j
public class ChuanyunScheduledTask {

    private final ChuanYunManager chuanYunManager;
    private final ChuanyunProjectDao chuanyunProjectDao;
    private final ChuanyunUserDao chuanyunUserDao;
    private final DingTalkProcessInstanceDao dingTalkProcessInstanceDao;
    private final ClawFeign clawFeign;

    public ChuanyunScheduledTask(ChuanYunManager chuanYunManager, ChuanyunProjectDao chuanyunProjectDao, ChuanyunUserDao chuanyunUserDao, DingTalkProcessInstanceDao dingTalkProcessInstanceDao, ClawFeign clawFeign) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunProjectDao = chuanyunProjectDao;
        this.chuanyunUserDao = chuanyunUserDao;
        this.dingTalkProcessInstanceDao = dingTalkProcessInstanceDao;
        this.clawFeign = clawFeign;
    }

    /**
     * 定时将氚云中的数据保存到manatee
     */
    @Scheduled(fixedDelay = 300000L)
    private void getDataFromChuanyun(){
        //获取用户数据
        getUserListFromChuanyun();
        //获取项目数据
        getProjectInformationFromChuanyun();
    }

    private void getUserListFromChuanyun() {
        ChuanyunFindAllDTO chuanyunFindAllDTO=chuanYunManager.findAll(ChuanyunUserDO.APP_CODE,ChuanyunUserDO.ACTION_NAME,ChuanyunUserDO.CONTROLLER);
        ObjectMapper objectMapper=new ObjectMapper();
        Integer totalCount=0;
        try {
            List<ChuanyunUserDO> userList=objectMapper.readValue(chuanyunFindAllDTO.getReturnData().getData(),new TypeReference<>(){});
            userList.forEach(chuanyunUserDO -> {
                if(StringUtils.isEmpty(chuanyunUserDO.getEmployeeNumber())){
                    ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),"");
                }else{
                    ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),chuanyunUserDO.getEmployeeNumber());
                }
            });
            totalCount=chuanyunUserDao.saveAll(userList).size();
        } catch (Exception e) {
            e.printStackTrace();
        }
        log.info("用户信息保存保存"+totalCount+"工号映射更新完成");
    }

    /**
     * 定时将钉钉同步的审批记录同步到氚云当中
     */
    @Scheduled(fixedDelay = 600000L)
    private void putProcessInstanceToChuanyun(){
    }

    /**
     * 定时将manatee中的信息同步到claw中
     */
    @Scheduled(fixedDelay = 600000L,initialDelay = 10000L)
    private void putDataToClaw(){
        putProjectToClaw();
    }

    /**
     * 将项目信息同步更新到U9系统中
     */
    private void putProjectToClaw(){
        //定时将isCreate为空的信息改为false
        var page=0;
        var totalPages=Integer.MAX_VALUE;

        while (page<totalPages){
            var pageable=PageRequest.of(page,20);
            var chuanyunProjectPage=chuanyunProjectDao.findAllByIsCreate(null,pageable);
            totalPages=chuanyunProjectPage.getTotalPages();
            chuanyunProjectPage.getContent().forEach(data->data.setIsCreate(false));
            chuanyunProjectDao.saveAll(chuanyunProjectPage.getContent());
            page++;
        }

        //同步项目数据
        page=0;
        totalPages=Integer.MAX_VALUE;
        while (page<totalPages){
            var pageable=PageRequest.of(page,20);
            var chuanyunProjectPage=chuanyunProjectDao.findAllByIsCreate(false,pageable);
            chuanyunProjectPage.getContent().forEach(chuanyunProjectDO -> {
                try{
                    send(chuanyunProjectDO);
                }catch (Exception exception){
                    log.error("保存到claw失败"+chuanyunProjectDO.toString());
                }
            });
            page++;
        }
    }

    /**
     * 调用feign发送数据
     * @param chuanyunProjectDO 项目数据
     */
    @Transactional(rollbackFor = Exception.class)
    protected void send(ChuanyunProjectDO chuanyunProjectDO){
        var responseEntity=clawFeign.saveChuanyunProject(ChuanyunProjectDO.toChuanyunProjectDTO(chuanyunProjectDO));
        if(responseEntity.getStatusCode().equals(HttpStatus.OK)){
            //如果已经是新写入claw服务需要修改是否为创建新数据的状态
            if (!chuanyunProjectDO.getIsCreate()){
                chuanyunProjectDO.setIsCreate(true);
                chuanyunProjectDao.save(chuanyunProjectDO);
            }
        }
    }

    /**
     * 定时更新项目信息
     */
    private void getProjectInformationFromChuanyun(){
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                Filter filter=Filter.instance(start,start+20,true);
                ChuanyunFindAllBizDTO<ChuanyunProjectDO> chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+20){
                    start+=20;
                }else{
                    flag=false;
                }
                //转化为POJO
                ObjectMapper objectMapper=new ObjectMapper();
                List<ChuanyunProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //更新缓存中的项目映射表
                result.forEach(projectBO -> ChuanYunConstant.PROJECT_CODE.put(projectBO.getProjectCode(),projectBO.getObjectId()));
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunProjectDO -> {
                    var tmpProjectDO=chuanyunProjectDao.findById(chuanyunProjectDO.getObjectId());
                    tmpProjectDO.ifPresent(projectDO -> chuanyunProjectDO.setIsCreate(projectDO.getIsCreate()));
                    //将projectManger和projectOwner替换为主键
                    if(!StringUtils.isEmpty(chuanyunProjectDO.getProjectManager())){
                        var projectManagerIds=new StringBuilder();
                        for (ChuanyunObject object:chuanyunProjectDO.getProjectManagerObject()){
                            projectManagerIds.append(object.getObjectId()).append(StringConstant.SEMICOLON);
                        }
                        chuanyunProjectDO.setProjectManager(projectManagerIds.toString());
                    }
                    if(!StringUtils.isEmpty(chuanyunProjectDO.getProjectOwner())){
                        chuanyunProjectDO.setProjectOwner(chuanyunProjectDO.getProjectOwnerObject().getObjectId());
                    }
                });
                chuanyunProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    /**
     * 将物料补发数据同步到氚云
     * 预计2020年4月份之后逐步将各个部门物料补发数据迁移到氚云进行审批，这个功能逐步废弃
     * 不能调用这个方法！！！！！
     */
    @Deprecated
    private void putMaterialResendInstance(){
        var currentPage=0;
        var totalPage=Integer.MAX_VALUE;

        var objectMapper=new ObjectMapper();
        objectMapper.registerModule(new JavaTimeModule());
        objectMapper.disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS);

        while (currentPage<totalPage){
            List<String> strings=new ArrayList<>();
            PageRequest pageRequest=PageRequest.of(currentPage,20);
            DingTalkProcessInstance dingTalkProcessInstance=new DingTalkProcessInstance();
            dingTalkProcessInstance.setProcessInstanceId(DingTalkConstant.PROCESS_CODE_MATERIAL_RESEND);
            dingTalkProcessInstance.setUpdatedInChuanyun(false);
            Page<DingTalkProcessInstance> pageList=dingTalkProcessInstanceDao.findAll(Example.of(dingTalkProcessInstance),pageRequest);
            if(totalPage==Integer.MAX_VALUE){
                totalPage=pageList.getTotalPages();
            }

            pageList.getContent().forEach(content->{
                if(content.getProcessInstance()==null){
                    log.info("观察异常内容"+content+"");
                }else {
                    if (content.getProcessInstanceId().equals(DingTalkConstant.PROCESS_CODE_MATERIAL_RESEND)){
                        MaterialResendBO materialResendBO = MaterialResendBO.fromDingTak(content.getProcessInstance());
                        try {
                            strings.add(objectMapper.writeValueAsString(materialResendBO));
                        } catch (JsonProcessingException e) {
                            log.error(e.getMessage());
                        }
                    }
                }
            });
            ChuanyunSaveAllDTO chuanyunSaveAllResponse= chuanYunManager.saveAll(MaterialResendBO.SCHEMA_CODE,strings,true);
            log.info("氚云保存物料补发数据结果的错误信息为 "+chuanyunSaveAllResponse.getErrorMessage());
            log.info(chuanyunSaveAllResponse+"");
            pageList.getContent().forEach(content->content.setUpdatedInChuanyun(true));
            log.info("开始更新钉钉审批数据上传状态");
            log.info(dingTalkProcessInstanceDao.saveAll(pageList.getContent())+"");
            log.info("已更新上传状态的钉钉审批数据");
            currentPage++;
        }
    }
}
